package net.sf.corn.gate;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * This annotation designed to use in conjunction with {@link GateService}
 * annotation. It defines name of the operation and operation's security details.
 * @author Serhat Dirik
 */
@Target(ElementType.METHOD)
@Retention(RetentionPolicy.RUNTIME)
public @interface GateOperation {
	/**
	 * Name of the operation, if not specified, the bounded methods name is used
	 * @return name of the operation
	 */
	String name() default "";
	/**
	 * Purpose of the operation
	 * @return operation's detailed description
	 */
	String description() default "";
	boolean authenticationRequiredToInspect() default false;
	boolean authenticationRequiredToUse() default false;
	boolean secureProtocolRequiredToInspect() default false;
	boolean secureProtocolRequiredToUse() default false;
	boolean remoteAccessEnabled() default true;
	
	String[] authorizedRolesToInspect() default {};
	String[] authorizedRolesToUse() default {};
}
