package net.sf.corn.gate;

import java.util.List;
import java.util.Set;

/**
 * Filters are combination of service targets and advice. Service 
 * operation calls that targeted by service targets in filter are 
 * intercepted and advice declarations inside the filter are called.
 * A Filter implementation must be registered to be activated. 
 *  
 * @author Serhat Dirik
 */
public interface IServiceFilter {

	/**
	 * Gives the name of the filter
	 * @return filter name
	 */
	public String getName();

	/**
	 * Gives the description of the filter
	 * @return a description string
	 */
	public String getDescription();

	/**
	 * Gives the set of filter targets 
	 * @return set of service targets 
	 */
	public Set<ServiceTargetDefinition> getServiceTargetDefinitions();

	/**
	 * Gives the advice list
	 * @return List of execution advice
	 */
	public List<IExecutionAdvice> getAdvice();

	/**
	 * Tells if the given service and operation is the target of this filter
	 * @param serviceName service name 
	 * @param operationName operation name 
	 * @return true if the given service and operation is targeted by this filter
	 */
	public boolean matches(String serviceName, String operationName);
}
