/*******************************************************************************
 * This JS Library contains tree classes to make JsonRpc Ajax calls through Gate  
 * @author Serhat Dirik 
 * @version 1.0
 ******************************************************************************/
// The class gives you a cross browser XMLHttpRequest object.
var XMLHttpRequestFactory = {
  getXmlHttpRequest: function() {
    var _xmlhttp=null;
    if (window.XMLHttpRequest){	// code for Mozilla, etc.
			try{
				_xmlhttp=new XMLHttpRequest();
			}catch(e){
				txt = "An error has occured while creating XMLHttpRequest \n";
				if(e.message) txt += e.message ;
				else txt += e ;
				alert(txt);
				throw new Error(txt);
			}
		}else if (window.ActiveXObject){// code for IE
			try {
	        	_xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
	        	_isBrowserIE=true;
	      	} catch(e) {
	        	try {
	          		_xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
	          		_isBrowserIE=true;
	        	} catch(e) {
	          		txt = "An error has occured while creating Microsoft XMLHttpRequest \n";
					if(e.message) txt += e.message ;
					else txt += e ;
					alert(txt);
					throw new Error(txt);
	        	}
	        }	
		}else if(window.createRequest){
			try {
				_xmlhttp = window.createRequest();
			} catch (e) {
				txt = "An error has occured while creating window.createRequest \n";
				if(e.message) txt += e.message ;
				else txt += e ;
				alert(txt);
				throw new Error(txt); 
			}

		}else{
			txt = "Your browser doesn't support XMLHttpSupport";
			alert(txt);
			throw new Error(txt);
		}
	 return _xmlhttp;
  }
}
//JsonRpcRequest class to send service requests
function JsonRpcRequest(url, user, password) {
	if (!url || url == null) {
		throw new Error("Invalid URL specified for JsonRpcGate");
	}
	var _url = url;
	var _user = user;
	var _password = password;
	var _XMLHttpRequestPool = new Array();
	var CHARSET = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	
	function _setUser(user){
		_user = user;
	}
	function _setPassword(password){
		_password = password;
	}
	
	function _resolveResult(resultText) {
		var obj;
		try {
			eval("obj = " + resultText);
		} catch (e) {
			obj ={};
			obj.error={};
			obj.error.errorCode = "595";
			obj.error.errorMsg = e.message;
			obj.error.errorName = "JsonEvaluationException";
		}
		return obj;

	}
	

	function _getXMLHTTPRequest() {
		var _xmlhttp = XMLHttpRequestFactory.getXmlHttpRequest();
		return _xmlhttp;
	}

	// escape a character
	function _escapeJSONChar(c) {
		if (c == "\"" || c == "\\")
			return "\\" + c;
		else if (c == "\b")
			return "\\b";
		else if (c == "\f")
			return "\\f";
		else if (c == "\n")
			return "\\n";
		else if (c == "\r")
			return "\\r";
		else if (c == "\t")
			return "\\t";
		var hex = c.charCodeAt(0).toString(16);
		if (hex.length == 1)
			return "\\u000" + hex;
		else if (hex.length == 2)
			return "\\u00" + hex;
		else if (hex.length == 3)
			return "\\u0" + hex;
		else
			return "\\u" + hex;
	}
	// encode a string into JSON format
	function _escapeJSONString(s) {
		var parts = s.split("");
		for ( var i = 0; i < parts.length; i++) {
			var c = parts[i];
			if (c == '"' || c == '\\' || c.charCodeAt(0) < 32
					|| c.charCodeAt(0) >= 128)
				parts[i] = _escapeJSONChar(parts[i]);
		}
		return "\"" + parts.join("") + "\"";
	}

	// Marshall objects to JSON format
	function _toJSON(o) {
		if (o == null) {
			return 'null';
		} else if (o.constructor == String) {
			return _escapeJSONString(o);
		} else if (o.constructor == Number) {
			return o.toString();
		} else if (o.constructor == Boolean) {
			return o.toString();
		} else if (o.constructor == Date) {
			return '{classHint: "java.util.Date", time: ' + o
					.valueOf() + '}';
		} else if (o.constructor == Array) {
			var v = [];
			for ( var i = 0; i < o.length; i++)
				v.push(_toJSON(o[i]));
			return "[" + v.join(", ") + "]";
		} else {
			var v = [];
			for (attr in o) {
				if (o[attr] == null)
					v.push("\"" + attr + "\": null");
				else if (typeof o[attr] == "function")
					; // skip
				else
					v.push(_escapeJSONString(attr) + ": " + _toJSON(o[attr]));
			}
			return "{" + v.join(", ") + "}";
		}
	}

	function _send(id, service,operation , parameters, headers, cb) {
		if (!service || service == null) {
			throw new Error('service is not specified!');
		}
		if (!operation || operation == null) {
			throw new Error('operation is not specified!');
		}
		if (cb && typeof cb != 'function') {
			throw new Error('Call Back must be a function!');
		}
		
		var request = {};		
		request.service = service;
		request.operation = operation;
		if(id && id != null){
			request.id = id;
		}
		if(headers && headers != null){
			request.header = headers;
		}
		if(parameters && parameters != null){
			request.params = parameters;
		}
		
		
		var asynch=true;
		if (arguments.length < 5 ||!cb || cb == null){
			asynch = false;
		} 
		var _xmlhttp = _getXMLHTTPRequest();
		
		var data = _toJSON(request);
		if(_user && _user != null){
			_xmlhttp.open("POST",_url,asynch,_user,_password);
		}else{
			_xmlhttp.open("POST",_url,asynch,_genUUID());
		}
		//_xmlhttp.withCredentials = false;
		//keep alive false
		_xmlhttp.setRequestHeader("Connection","close");
		_xmlhttp.setRequestHeader("Cache-Control","no-cache");
		_xmlhttp.setRequestHeader("Pragma","no-cache");
		if (cb) {
			_xmlhttp.onreadystatechange = function() {
				if (_xmlhttp.readyState == 4) {
					var response;
					if (_xmlhttp.status == 200) {
						try {
							response = _resolveResult(_xmlhttp.responseText); 
						} catch (e) {
							throw e;
						}
					} else {
						response = {};
						response.error={};
						response.error.errorCode = _xmlhttp.status.toString();
						response.error.errorMsg = _xmlhttp.statusText;
						response.error.errorName = "XMLHttpException";
					}
					
					cb(response);
				}
			}
		} else {
			_xmlhttp.onreadystatechange = function() {
			};
		}
		_xmlhttp.send(data);
		if (!cb) {
			var response = null;
			if (_xmlhttp.status == 200) {
				try {
					response = _resolveResult(_xmlhttp.responseText); 
				} catch (e) {
					throw e;
				}
			} else {
				response = {};
				response.error={};
				response.error.errorCode = _xmlhttp.status.toString();
				response.error.errorMsg = _xmlhttp.statusText;
				response.error.errorName = "XMLHttpException";
			}
			return response;
		}
	}
	
	function _rand(max){
		return Math.floor(Math.random() * (max + 1));
	};
	
	function _genUUID(){
		var dc = new Date();
		var ctime = dc.getTime().toString(16);
		var uuid = new Array;
		for (var i = 0; i < 16; i++) {
				uuid[i] = CHARSET.charAt(_rand(62));
		}
		return ctime+"-"+ uuid.join("");
	};
	
	
	//Public functions 
	this.send = _send;
	this.toJSON =_toJSON;
	this.genUUID= _genUUID;
	this.setUser=_setUser;
	this.setPassword=_setPassword;
}
 

 

